<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Auth extends MY_Controller
{

    function __construct()
    {
        parent::__construct(false, false);

        // Load required CI libraries and helpers.
        $this->load->helper('form');

        // Redirect users logged in via password (However, not 'Remember me' users, as they may wish to login properly).
        if ($this->flexi_auth->is_logged_in()) {
            // Preserve any flashdata messages so they are passed to the redirect page.
            if ($this->session->flashdata('message')) {
                $this->session->keep_flashdata('message');
            }

            // Redirect logged in admins (For security, admin users should always sign in via Password rather than 'Remember me'.
            redirect();
        }

        // Define a global variable to store data that is then used by the end view page.
        $this->data = null;
    }

    public function _remap($method, $params = array())
    {
        switch ($method) {
            case lang('link_activate_account'):
                return call_user_func_array(array($this, 'activate_account'), $params);
                break;
            case lang('link_manual_reset_forgotten_password'):
                return call_user_func_array(array($this, 'manual_reset_forgotten_password'), $params);
                break;
            case lang('link_forgotten_password'):
                return call_user_func_array(array($this, 'forgotten_password'), $params);
                break;
            case lang('link_register_account'):
                return call_user_func_array(array($this, 'register_account'), $params);
                break;
            case lang('link_resend_activation_token'):
                return call_user_func_array(array($this, 'resend_activation_token'), $params);
                break;
            default:
                $this->index();
        }
    }

    public function index()
    {
        $this->login();
    }

    function login()
    {
        // If 'Login' form has been submited, attempt to log the user in.
        if ($this->input->post('login_user')) {
            $this->load->model('auth_model');
            $this->auth_model->login();
        }

        // CAPTCHA Example
        // Check whether there are any existing failed login attempts from the users ip address and whether those attempts have exceeded the defined threshold limit.
        // If the user has exceeded the limit, generate a 'CAPTCHA' that the user must additionally complete when next attempting to login.
        if ($this->flexi_auth->ip_login_attempts_exceeded()) {
            /**
             * reCAPTCHA
             * http://www.google.com/recaptcha
             * To activate reCAPTCHA, ensure the 'recaptcha()' function below is uncommented and then comment out the 'math_captcha()' function further below.
             *
             * A boolean variable can be passed to 'recaptcha()' to set whether to use SSL or not.
             * When displaying the captcha in a view, reCAPTCHA generates all html required for display.
             *
             * Note: To use this example, you will also need to enable the recaptcha examples in 'models/demo_auth_model.php', and 'views/demo/login_view.php'.
             */
            $this->data['captcha'] = $this->flexi_auth->recaptcha(FALSE);

            /**
             * flexi auths math CAPTCHA
             * Math CAPTCHA is a basic CAPTCHA style feature that asks users a basic maths based question to validate they are indeed not a bot.
             * For flexibility on CAPTCHA presentation, the 'math_captcha()' function only generates a string of the equation, see the example below.
             *
             * To activate math_captcha, ensure the 'math_captcha()' function below is uncommented and then comment out the 'recaptcha()' function above.
             *
             * Note: To use this example, you will also need to enable the math_captcha examples in 'models/demo_auth_model.php', and 'views/demo/login_view.php'.
             */
            # $this->data['captcha'] = $this->flexi_auth->math_captcha(FALSE);
        }

        $this->data['title'] = "Prihlásenie";
        $this->master_view('user/login', $this->data, 'login');
    }

    /**
     * register_account
     * User registration page used by all new users wishing to create an account.
     * Note: This page is only accessible to users who are not currently logged in, else they will be redirected.
     */
    function register_account()
    {
        // Redirect user away from registration page if already logged in.
        if ($this->flexi_auth->is_logged_in()) {
            redirect(lang('link_auth'));
        } // If 'Registration' form has been submitted, attempt to register their details as a new account.
        else if ($this->input->post('register_user')) {
            $this->load->model('auth_model');
            $this->auth_model->register_account();
        }

        $this->data['title'] = "Registrácia";
        $this->master_view('user/register', $this->data, 'reg');
    }

    ###++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++###
    // Account Activation
    ###++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++###

    /**
     * activate_account
     * User account activation via email.
     * The default setup of this demo requires that new account registrations must be authenticated via email before the account is activated.
     * In this demo, this page is accessed via an activation link in the 'views/includes/email/activate_account.tpl.php' email template.
     */
    function activate_account($user_id, $token = FALSE)
    {
        // The 3rd activate_user() parameter verifies whether to check '$token' matches the stored database value.
        // This should always be set to TRUE for users verifying their account via email.
        // Only set this variable to FALSE in an admin environment to allow activation of accounts without requiring the activation token.
        $this->flexi_auth->activate_user($user_id, $token, TRUE);

        // Save any public status or error messages (Whilst suppressing any admin messages) to CI's flash session data.
        $this->session->set_flashdata('message', $this->flexi_auth->get_messages());

        redirect(lang('link_auth'));
    }

    /**
     * resend_activation_token
     * Resend user an activation token via email.
     * If a user has not received/lost their account activation email, they can request a new activation email to be sent to them.
     * In this demo, this page is accessed via a link on the login page.
     */
    function resend_activation_token()
    {
        // If the 'Resend Activation Token' form has been submitted, resend the user an account activation email.
        if ($this->input->post('send_activation_token')) {
            $this->load->model('auth_model');
            $this->auth_model->resend_activation_token();
        }

        $this->data['title'] = "Preposlať aktivačný kľúč";
        $this->master_view('user/resend_activation_token', $this->data);
    }

    ###++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++###
    // Forgotten Password
    ###++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++###

    /**
     * forgotten_password
     * Send user an email to verify their identity. Via a unique link in this email, the user is redirected to the site so they can then reset their password.
     * In this demo, this page is accessed via a link on the login page.
     *
     * Note: This is step 1 of an example of allowing users to reset a forgotten password manually.
     * See the auto_reset_forgotten_password() function below for an example of directly emailing the user a new randomised password.
     */
    function forgotten_password()
    {
        // If the 'Forgotten Password' form has been submitted, then email the user a link to reset their password.
        if ($this->input->post('send_forgotten_password')) {
            $this->load->model('auth_model');
            $this->auth_model->forgotten_password();
        }

        $this->data['title'] = "Zabudnuté heslo";
        $this->master_view('user/forgot_password', $this->data);
    }

    /**
     * manual_reset_forgotten_password
     * This is step 2 (The last step) of an example of allowing users to reset a forgotten password manually.
     * See the auto_reset_forgotten_password() function below for an example of directly emailing the user a new randomised password.
     * In this demo, this page is accessed via a link in the 'views/includes/email/forgot_password.tpl.php' email template, which must be set to 'auth/manual_reset_forgotten_password/...'.
     */
    function manual_reset_forgotten_password($user_id = FALSE, $token = FALSE)
    {
        // If the 'Change Forgotten Password' form has been submitted, then update the users password.
        if ($this->input->post('change_forgotten_password')) {
            $this->load->model('auth_model');
            $this->auth_model->manual_reset_forgotten_password($user_id, $token);
        }

        $this->data['title'] = "Zmena zabudnutého hesla";
        $this->master_view('user/forgot_password_update', $this->data);
    }

    /**
     * auto_reset_forgotten_password
     * This is an example of automatically reseting a users password as a randomised string that is then emailed to the user.
     * See the manual_reset_forgotten_password() function above for the manual method of changing a forgotten password.
     * In this demo, this page is accessed via a link in the 'views/includes/email/forgot_password.tpl.php' email template, which must be set to 'auth/auto_reset_forgotten_password/...'.
     */
    function auto_reset_forgotten_password($user_id = FALSE, $token = FALSE)
    {
        // forgotten_password_complete() will validate the token exists and reset the password.
        // To ensure the new password is emailed to the user, set the 4th argument of forgotten_password_complete() to 'TRUE' (The 3rd arg manually sets a new password so set as 'FALSE').
        // If successful, the password will be reset and emailed to the user.
        $this->flexi_auth->forgotten_password_complete($user_id, $token, FALSE, TRUE);

        // Set a message to the CI flashdata so that it is available after the page redirect.
        $this->session->set_flashdata('message', $this->flexi_auth->get_messages());

        redirect(lang('link_auth'));
    }

    ###++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++###
    // Logout
    ###++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++###

    /**
     * logout
     * This example logs the user out of all sessions on all computers they may be logged into.
     * In this demo, this page is accessed via a link on the demo header once a user is logged in.
     */
    function logout()
    {
        // By setting the logout functions argument as 'TRUE', all browser sessions are logged out.
        $this->flexi_auth->logout(TRUE);

        // Set a message to the CI flashdata so that it is available after the page redirect.
        $this->session->set_flashdata('message', $this->flexi_auth->get_messages());

        redirect(lang('link_auth'));
    }

}